from chameleon.core import translation
from chameleon.core import config
from chameleon.core import utils

import chameleon.zpt.language

class ZopePageTemplateElement(chameleon.zpt.language.ZopePageTemplateElement):
    meta_interpolation = utils.attribute(
        utils.meta_attr('interpolation'), default="false")

class XHTMLElement(chameleon.zpt.language.XHTMLElement):
    meta_interpolation = utils.attribute(
        utils.meta_attr('interpolation'), default="false")

class TALElement(chameleon.zpt.language.TALElement):
    meta_interpolation = utils.attribute(
        utils.meta_attr('interpolation'), default="false")

class METALElement(chameleon.zpt.language.METALElement):
    meta_interpolation = utils.attribute(
        utils.meta_attr('interpolation'), default="false")

class MetaElement(chameleon.zpt.language.MetaElement):
    meta_interpolation = utils.attribute(
        utils.meta_attr('interpolation'), default="false")

class Parser(chameleon.zpt.language.Parser):
    """Zope Page Template parser."""

    element_mapping = {
        config.XHTML_NS: {None: XHTMLElement},
        config.META_NS: {None: MetaElement},
        config.TAL_NS: {None: TALElement},
        config.METAL_NS: {None: METALElement}}

    fallback = XHTMLElement
    default_expression = 'path'



import errno
import os.path

from z3c.pt.pagetemplate import PageTemplateFile
from z3c.pt.texttemplate import TextTemplateFile

from chameleon.core import loader

class TemplateLoader(loader.TemplateLoader):
    def load_page(self, filename):
        return self.load(filename, PageTemplateFile)

    def load_text(self, filename):
        return self.load(filename, TextTemplateFile)




import os
import sys
import compiler

from zope import i18n
from zope import component

from chameleon.core import types
from chameleon.core import config
from chameleon.core import codegen
from chameleon.core import clauses
from chameleon.core import generation
from chameleon.core import utils

from chameleon.zpt import template
from chameleon.zpt.interfaces import IExpressionTranslator

from z3c.pt import language

_marker = object()
_expr_cache = {}

def evaluate_expression(pragma, expr):
    key = "%s(%s)" % (pragma, expr)
    try:
        symbol_mapping, parts, source = _expr_cache[key]
    except KeyError:
        translator = component.getUtility(IExpressionTranslator, name=pragma)
        parts = translator.tales(expr)
        stream = generation.CodeIO(symbols=config.SYMBOLS)
        assign = clauses.Assign(parts, 'result')
        assign.begin(stream)
        assign.end(stream)
        source = stream.getvalue()

        symbol_mapping = parts.symbol_mapping.copy()
        if isinstance(parts, types.parts):
            for value in parts:
                symbol_mapping.update(value.symbol_mapping)    

        _expr_cache[key] = symbol_mapping, parts, source

    # acquire template locals and update with symbol mapping
    frame = sys._getframe()
    while frame.f_locals.get('econtext', _marker) is _marker:
        frame = frame.f_back
        if frame is None:
            raise RuntimeError, "Can't locate template frame."

    _locals = frame.f_locals
    _locals.update(symbol_mapping)
    
    # to support dynamic scoping (like in templates), we must
    # transform the code to take the scope locals into account; for
    # efficiency, this is cached for reuse
    code_cache_key = key, tuple(_locals)
    try:
        code = _expr_cache[code_cache_key]
    except KeyError:
        suite = codegen.Suite(source, _locals)
        code = compiler.compile(
            suite.source, 'dynamic_path_expression.py', 'exec')
        _expr_cache[code_cache_key] = code

    # execute code and return evaluation
    exec code in codegen.lookup_globals.copy(), _locals
    return _locals['result']

def evaluate_path(expr):
    return evaluate_expression('path', expr)

def evaluate_exists(expr):
    try:
        return evaluate_expression('exists', expr)
    except NameError:
        return False

class BaseTemplate(template.PageTemplate):
    default_parser = language.Parser()
    version = 1
    
    def bind(self, ob, request=None, macro=None, global_scope=True):
        def render(target_language=None, **kwargs):
            context = self._pt_get_context(ob, request, kwargs)
            if target_language is None:
                try:
                    target_language = i18n.negotiate(
                        request or context.get('request'))
                except:
                    target_language = None

            context['target_language'] = target_language
            context["econtext"] = utils.econtext(context)

            if macro is None:
                return self.render(**context)
            else:
                return self.render_macro(
                    macro, global_scope=global_scope, parameters=context)

        return BoundPageTemplate(self, render)

    def __call__(self, _ob=None, **kwargs):
        bound_pt = self.bind(_ob)
        return bound_pt(**kwargs)

    def _pt_get_context(self, instance, request, kwargs):
        return dict(
            options=kwargs,
            request=request,
            template=self,
            path=evaluate_path,
            exists=evaluate_exists,
            nothing=None,
            modules=sys.modules)

class BaseTemplateFile(BaseTemplate, template.PageTemplateFile):
    """If ``filename`` is a relative path, the module path of the
    class where the instance is used to get an absolute path."""

    def __init__(self, filename, path=None, content_type=None, **kwargs):
        if path is not None:
            filename = os.path.join(path, filename)

        if not os.path.isabs(filename):	       
            for depth in (1, 2):	       
                frame = sys._getframe(depth)	 
                package_name = frame.f_globals['__name__']	 
 	 
                if package_name != self.__module__:	 
                    break	 
 	 
            module = sys.modules[package_name]
            try:	 
                path = module.__path__[0]	 
            except AttributeError:	 
                path = module.__file__	 
                path = path[:path.rfind(os.sep)]	 
 	 
            filename = path + os.sep + filename

        template.PageTemplateFile.__init__(
            self, filename, **kwargs)

class PageTemplate(BaseTemplate):
    """Page Templates using TAL, TALES, and METAL.

    This class is suitable for standalone use or class
    property. Keyword-arguments are passed into the template as-is.

    Initialize with a template string."""

    version = 1
    
    def __get__(self, instance, type):
        if instance is not None:
            return self.bind(instance)
        return self
    
class PageTemplateFile(BaseTemplateFile, PageTemplate):
    """Page Templates using TAL, TALES, and METAL.

    This class is suitable for standalone use or class
    property. Keyword-arguments are passed into the template as-is.

    Initialize with a filename."""

class ViewPageTemplate(PageTemplate):
    """Template class suitable for use with a Zope browser view; the
    variables ``view``, ``context`` and ``request`` variables are
    brought in to the local scope of the template automatically, while
    keyword arguments are passed in through the ``options``
    dictionary. Note that the default expression type for this class
    is 'path' (standard Zope traversal)."""
    
    def _pt_get_context(self, view, request, kwargs):
        context = kwargs.get('context')
        if context is None:
            context = view.context
        request = request or kwargs.get('request') or view.request
        return dict(
            view=view,
            context=context,
            request=request,
            template=self,
            path=evaluate_path,
            exists=evaluate_exists,
            options=kwargs,
            nothing=None,
            modules=sys.modules)

    def __call__(self, _ob=None, context=None, request=None, **kwargs):
        kwargs.setdefault('context', context)
        kwargs.setdefault('request', request)
        return super(ViewPageTemplate, self).__call__(
            _ob=_ob, **kwargs)

class ViewPageTemplateFile(ViewPageTemplate, PageTemplateFile):
    """If ``filename`` is a relative path, the module path of the
    class where the instance is used to get an absolute path."""

class BoundPageTemplate(object):
    """When a page template class is used as a property, it's bound to
    the class instance on access, which is implemented using this
    helper class."""
    
    def __init__(self, pt, render):
        object.__setattr__(self, 'im_self', pt)
        object.__setattr__(self, 'im_func', render)
        
    macros = property(lambda self: self.im_self.macros)
    filename = property(lambda self: self.im_self.filename)

    def __call__(self, *args, **kw):
        return self.im_func(**kw)

    def __setattr__(self, name, v):
        raise AttributeError("Can't set attribute", name)

    def __repr__(self):
        return "<%s.Bound%s %r>" % (
            type(self.im_self).__module__,
            type(self.im_self).__name__, self.filename)



import zope.component
from zope.traversing.interfaces import IPathAdapter

class AdapterNamespaces(object):
    """Simulate tales function namespaces with adapter lookup.

    When we are asked for a namespace, we return an object that
    actually computes an adapter when called:

    To demonstrate this, we need to register an adapter:

      >>> def adapter1(ob):
      ...     return 1
      >>> zope.component.getGlobalSiteManager().registerAdapter(
      ...     adapter1, [zope.interface.Interface], IPathAdapter, 'a1')

    Now, with this adapter in place, we can try out the namespaces:

      >>> ob = object()
      >>> namespaces = AdapterNamespaces()
      >>> namespace = namespaces['a1']
      >>> namespace(ob)
      1
      >>> namespace = namespaces['a2']
      >>> namespace(ob)
      Traceback (most recent call last):
      ...
      KeyError: 'a2'
    """

    def __init__(self):
        self.namespaces = {}

    def __getitem__(self, name):
        namespace = self.namespaces.get(name)
        if namespace is None:
            def namespace(object):
                try:
                    return zope.component.getAdapter(object, IPathAdapter, name)
                except zope.component.ComponentLookupError:
                    raise KeyError(name)

            self.namespaces[name] = namespace
        return namespace


    def registerFunctionNamespace(self, namespacename, namespacecallable):
        """Register a function namespace

        namespace - a string containing the name of the namespace to
                    be registered

        namespacecallable - a callable object which takes the following
                            parameter:

                            context - the object on which the functions
                                      provided by this namespace will
                                      be called

                            This callable should return an object which
                            can be traversed to get the functions provided
                            by the this namespace.

        example:

           class stringFuncs(object):

              def __init__(self,context):
                 self.context = str(context)

              def upper(self):
                 return self.context.upper()

              def lower(self):
                 return self.context.lower()

            engine.registerFunctionNamespace('string',stringFuncs)
        """
        self.namespaces[namespacename] = namespacecallable


    def getFunctionNamespace(self, namespacename):
        """ Returns the function namespace """
        return self.namespaces[namespacename]

try:
    # If zope.app.pagetemplates is available, use the adapter
    # registered with the main zope.app.pagetemplates engine so that
    # we don't need to re-register them.
    from zope.app.pagetemplates.engine import Engine
    function_namespaces = Engine.namespaces
except (ImportError, AttributeError):
    function_namespaces = AdapterNamespaces()




def render(_init, template, xincludes, request, context, nothing, options, view, target_language=None):
	_out, _write = _init.initialize_stream()
	_attributes, repeat = _init.initialize_tal()
	_scope = {}
	_domain = None

	# _path(view, request, True, )
	_write('<div>\n  ')
	_content = _path(view, request, True, )
	# not (_content is None)
	_tmp1 = not (_content is None)
	if _tmp1:
		_write('<span>')
	# _content
	_tmp2 = _content
	_tmp = _tmp2
	if _tmp is not None:
		if not isinstance(_tmp, (str, unicode)):
			_tmp = str(_tmp)
		if '&' in _tmp:
			_tmp = _tmp.replace('&', '&amp;')
		if '<' in _tmp:
			_tmp = _tmp.replace('<', '&lt;')
		if '>' in _tmp:
			_tmp = _tmp.replace('>', '&gt;')
		if '"' in _tmp:
			_tmp = _tmp.replace('"', '&quot;')
		_write(_tmp)
	_validate(_tmp)
	if _tmp1:
		_write('</span>')
	# _path(context, request, True, )
	_write('\n  ')
	_content = _path(context, request, True, )
	# not (_content is None)
	_tmp1 = not (_content is None)
	if _tmp1:
		_write('<span>')
	# _content
	_tmp2 = _content
	_tmp = _tmp2
	if _tmp is not None:
		if not isinstance(_tmp, (str, unicode)):
			_tmp = str(_tmp)
		if '&' in _tmp:
			_tmp = _tmp.replace('&', '&amp;')
		if '<' in _tmp:
			_tmp = _tmp.replace('<', '&lt;')
		if '>' in _tmp:
			_tmp = _tmp.replace('>', '&gt;')
		if '"' in _tmp:
			_tmp = _tmp.replace('"', '&quot;')
		_write(_tmp)
	_validate(_tmp)
	if _tmp1:
		_write('</span>')
	# _path(request, request, True, )
	_write('\n  ')
	_content = _path(request, request, True, )
	# not (_content is None)
	_tmp1 = not (_content is None)
	if _tmp1:
		_write('<span>')
	# _content
	_tmp2 = _content
	_tmp = _tmp2
	if _tmp is not None:
		if not isinstance(_tmp, (str, unicode)):
			_tmp = str(_tmp)
		if '&' in _tmp:
			_tmp = _tmp.replace('&', '&amp;')
		if '<' in _tmp:
			_tmp = _tmp.replace('<', '&lt;')
		if '>' in _tmp:
			_tmp = _tmp.replace('>', '&gt;')
		if '"' in _tmp:
			_tmp = _tmp.replace('"', '&quot;')
		_write(_tmp)
	_validate(_tmp)
	if _tmp1:
		_write('</span>')
	# _path(options, request, True, 'test')
	_write('\n  ')
	_content = _path(options, request, True, 'test')
	# not (_content is None)
	_tmp1 = not (_content is None)
	if _tmp1:
		_write('<span>')
	# _content
	_tmp2 = _content
	_tmp = _tmp2
	if _tmp is not None:
		if not isinstance(_tmp, (str, unicode)):
			_tmp = str(_tmp)
		if '&' in _tmp:
			_tmp = _tmp.replace('&', '&amp;')
		if '<' in _tmp:
			_tmp = _tmp.replace('<', '&lt;')
		if '>' in _tmp:
			_tmp = _tmp.replace('>', '&gt;')
		if '"' in _tmp:
			_tmp = _tmp.replace('"', '&quot;')
		_write(_tmp)
	_validate(_tmp)
	if _tmp1:
		_write('</span>')
	_write('\n')
	_write('</div>')
	_write('\n')

	return _out.getvalue()
